import { MatchRequest } from '../domain/dtos/match.request';
import { Geography } from '../domain/entities/geography';
import { Person } from '../domain/entities/person';
import { Requirement } from '../domain/entities/requirement';
import { Role } from '../domain/types/role.enum';
import { AlgorithmSettings } from '../domain/entities/algorithm-settings';
import { Time } from '../domain/entities/time';
import { IDefaultParams } from '../domain/types/default-params.type';

export class MatchQuery {
  private readonly _matchRequest: MatchRequest;
  private readonly _defaultParams: IDefaultParams;
  person: Person;
  roles: Array<Role>;
  time: Time;
  geography: Geography;
  exclusions: Array<number>;
  requirement: Requirement;
  algorithmSettings: AlgorithmSettings;

  constructor(matchRequest: MatchRequest, defaultParams: IDefaultParams) {
    this._matchRequest = matchRequest;
    this._defaultParams = defaultParams;
    this._setPerson();
    this._setRoles();
    this._setTime();
    this._setGeography();
    this._setRequirement();
    this._setAlgorithmSettings();
    this._setExclusions();
  }

  _setPerson() {
    this.person = new Person(
      this._matchRequest,
      this._defaultParams.DEFAULT_IDENTIFIER,
      this._defaultParams.MARGIN_DURATION,
    );
    this.person.init();
  }

  _setRoles() {
    this.roles = [];
    if (this._matchRequest.driver) this.roles.push(Role.DRIVER);
    if (this._matchRequest.passenger) this.roles.push(Role.PASSENGER);
    if (this.roles.length == 0) this.roles.push(Role.PASSENGER);
  }

  _setTime() {
    this.time = new Time(
      this._matchRequest,
      this._defaultParams.MARGIN_DURATION,
      this._defaultParams.VALIDITY_DURATION,
    );
    this.time.init();
  }

  _setGeography() {
    this.geography = new Geography(
      this._matchRequest,
      this._defaultParams.DEFAULT_TIMEZONE,
    );
    this.geography.init();
  }

  _setRequirement() {
    this.requirement = new Requirement(
      this._matchRequest,
      this._defaultParams.DEFAULT_SEATS,
    );
  }

  _setAlgorithmSettings() {
    this.algorithmSettings = new AlgorithmSettings(
      this._matchRequest,
      this._defaultParams.DEFAULT_ALGORITHM_SETTINGS,
      this.time.frequency,
    );
  }

  _setExclusions() {
    this.exclusions = [];
    if (this._matchRequest.identifier)
      this.exclusions.push(this._matchRequest.identifier);
    if (this._matchRequest.exclusions)
      this.exclusions.push(...this._matchRequest.exclusions);
  }
}
